@@ -1,10 +1,25 @@
-# -*- coding: utf-8 -*-
+# Copyright (c) 2010-2013 by Yaco Sistemas <goinnn@gmail.com> or <pmartin@yaco.es>
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with this programe.  If not, see <http://www.gnu.org/licenses/>.
+
 from django import template
 from django.db import models
 from django.db.models.fields import FieldDoesNotExist
 from django.db.models.fields.related import ForeignKey, ManyToManyField
 from django.conf import settings
 
+from inplaceeditform import settings as inplace_settings
 from inplaceeditform.adaptors import ADAPTOR_INPLACEEDIT as DEFAULT_ADAPTOR_INPLACEEDIT
 
 has_transmeta = False
@@ -69,18 +84,30 @@ def get_adaptor_class(adaptor=None, obj=None, field_name=None):
         except FieldDoesNotExist:
             if has_transmeta:
                 field = obj._meta.get_field_by_name(transmeta.get_real_fieldname(field_name))[0]
-        if isinstance(field, models.CharField):
+        if isinstance(field, models.URLField):
+            adaptor = 'url'
+        elif isinstance(field, models.CharField):
             adaptor = 'text'
             if getattr(field, 'choices', None):
                 adaptor = 'choices'
         elif isinstance(field, models.TextField):
             adaptor = 'textarea'
+        elif isinstance(field, models.NullBooleanField):
+            adaptor = 'nullboolean'
         elif isinstance(field, models.BooleanField):
             adaptor = 'boolean'
         elif isinstance(field, models.DateTimeField):
             adaptor = 'datetime'
         elif isinstance(field, models.DateField):
             adaptor = 'date'
+        elif isinstance(field, models.TimeField):
+            adaptor = 'time'
+        elif isinstance(field, models.IntegerField):
+            adaptor = 'integer'
+        elif isinstance(field, models.FloatField):
+            adaptor = 'float'
+        elif isinstance(field, models.DecimalField):
+            adaptor = 'decimal'
         elif isinstance(field, ForeignKey):
             adaptor = 'fk'
         elif isinstance(field, ManyToManyField):
@@ -90,9 +117,8 @@ def get_adaptor_class(adaptor=None, obj=None, field_name=None):
         elif isinstance(field, models.FileField):
             adaptor = 'file'
     from inplaceeditform.fields import BaseAdaptorField
-    path_adaptor = adaptor and ((getattr(settings, 'ADAPTOR_INPLACEEDIT', None) and
-                                 settings.ADAPTOR_INPLACEEDIT.get(adaptor, None)) or
-                                 (DEFAULT_ADAPTOR_INPLACEEDIT.get(adaptor, None)))
+    path_adaptor = adaptor and (inplace_settings.ADAPTOR_INPLACEEDIT.get(adaptor, None) or
+                                DEFAULT_ADAPTOR_INPLACEEDIT.get(adaptor, None))
     if not path_adaptor and adaptor:
         return get_adaptor_class(obj=obj, field_name=field_name)
     elif not path_adaptor:
